@@ -65,6 +65,8 @@ Ready?  Ok...
     ### Change how nameless message-component files are named:
     $parser->output_prefix("msg");
 
+    ### Put temporary files somewhere else
+    $parser->tmp_dir("/var/tmp/mytmpdir");
 
 =head2 Examples of error recovery
 
@@ -88,7 +90,7 @@ Ready?  Ok...
 
 =head2 Examples of parser options
 
-    ### Automatically attempt to RFC-1522-decode the MIME headers?
+    ### Automatically attempt to RFC 2047-decode the MIME headers?
     $parser->decode_headers(1);             ### default is false
 
     ### Parse contained "message/rfc822" objects as nested MIME streams?
@@ -121,8 +123,6 @@ that parse MIME streams into MIME::Entity objects.
 
 #------------------------------
 
-# We require the new FileHandle methods, and a non-buggy version
-# of FileHandle->new_tmpfile:
 require 5.004;
 
 ### Pragmas:
@@ -130,11 +130,7 @@ use strict;
 use vars (qw($VERSION $CAT $CRLF));
 
 ### Built-in modules:
-use FileHandle ();
-use IO::Wrap;
-use IO::Scalar       1.117;
 use IO::ScalarArray  1.114;
-use IO::Lines        1.108;
 use IO::File;
 use IO::InnerFile;
 use File::Spec;
@@ -184,7 +180,7 @@ package MIME::Parser;
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "5.419";
+$VERSION = "5.427";
 
 ### How to catenate:
 $CAT = '/bin/cat';
@@ -247,13 +243,12 @@ sub init {
     $self->{MP5_DecodeBodies}    = 1;
     $self->{MP5_Interface}       = {};
     $self->{MP5_ParseNested}     = 'NEST';
-    $self->{MP5_Tmp}             = undef;
-    $self->{MP5_TmpRecycling}    = 1;
     $self->{MP5_TmpToCore}       = 0;
     $self->{MP5_IgnoreErrors}    = 1;
     $self->{MP5_UseInnerFiles}   = 0;
     $self->{MP5_UUDecode}        = 0;
     $self->{MP5_MaxParts}        = -1;
+    $self->{MP5_TmpDir}          = undef;
 
     $self->interface(ENTITY_CLASS => 'MIME::Entity');
     $self->interface(HEAD_CLASS   => 'MIME::Head');
@@ -279,8 +274,8 @@ sub init_parse {
     $self->{MP5_Results} = new MIME::Parser::Results;
 
     $self->{MP5_Filer}->results($self->{MP5_Results});
+    $self->{MP5_Filer}->purgeable([]);
     $self->{MP5_Filer}->init_parse();
-    $self->{MP5_Filer}->purgeable([]);   ### just to be safe
     $self->{MP5_NumParts} = 0;
     1;
 }
@@ -307,7 +302,7 @@ sub init_parse {
 
 I<Instance method.>
 Controls whether the parser will attempt to decode all the MIME headers
-(as per RFC-1522) the moment it sees them.  B<This is not advisable
+(as per RFC 2047) the moment it sees them.  B<This is not advisable
 for two very important reasons:>
 
 =over
@@ -676,7 +671,7 @@ sub process_header {
     @headlines and
 	$self->error("couldn't parse head; error near:\n",@headlines);
 
-    ### If desired, auto-decode the header as per RFC-1522.
+    ### If desired, auto-decode the header as per RFC 2047
     ###    This shouldn't affect non-encoded headers; however, it will decode
     ###    headers with international characters.  WARNING: currently, the
     ###    character-set information is LOST after decoding.
@@ -704,7 +699,7 @@ sub process_multipart {
     $self->debug("process_multipart...");
 
     ### Get actual type and subtype from the header:
-    my ($type, $subtype) = (split('/', $head->mime_type), "");
+    my ($type, $subtype) = (split('/', $head->mime_type, -1), '');
 
     ### If this was a type "multipart/digest", then the RFCs say we
     ### should default the parts to have type "message/rfc822".
@@ -802,8 +797,7 @@ sub process_singlepart {
 	}
 	else {
 	    $self->debug("using temp file");
-	    $ENCODED = $self->new_tmpfile($self->{Tmp});
-	    $self->{Tmp} = $ENCODED if $self->{TmpRecycle};
+	    $ENCODED = $self->new_tmpfile();
 	}
 
 	### Read encoded body until boundary (or EOF)...
@@ -926,10 +920,10 @@ sub hunt_for_uuencode {
     $self->whine("Found a $how_encoded attachment");
     my $pre;
     while (1) {
-	my @bin_data;
+	my $bin_data = '';
 
 	### Try next part:
-	my $out = IO::ScalarArray->new(\@bin_data);
+	my $out = IO::File->new(\$bin_data, '>:');
 	eval { $decoder->decode($ENCODED, $out) }; last if $@;
 	my $preamble = $decoder->last_preamble;
 	my $filename = $decoder->last_filename;
@@ -965,7 +959,7 @@ sub hunt_for_uuencode {
 	    $bin_ent->bodyhandle($self->new_body_for($bin_ent->head));
 	    $bin_ent->bodyhandle->binmode(1) or die "$ME: can't set to binmode: $!";
 	    my $io = $bin_ent->bodyhandle->open("w") or die "$ME: can't create: $!";
-	    $io->print(@bin_data) or die "$ME: can't print: $!";
+	    $io->print($bin_data) or die "$ME: can't print: $!";
 	    $io->close or die "$ME: can't close: $!";
 	    push @parts, $bin_ent;
 	}
@@ -1057,7 +1051,7 @@ sub process_part {
     if (not defined $head) {
        $self->debug("bogus empty part");
        $head = $self->interface('HEAD_CLASS')->new;
-       $head->mime_type('text/plain; charset=US-ASCII');
+       $head->mime_type('text/plain');
        $ent->head($head);
        $ent->bodyhandle($self->new_body_for($head));
        $ent->bodyhandle->open("w")->close or die "$ME: can't close: $!";
@@ -1071,7 +1065,7 @@ sub process_part {
     $head->mime_type($p{Retype}) if $p{Retype};
 
     ### Get the MIME type and subtype:
-    my ($type, $subtype) = (split('/', $head->mime_type), '');
+    my ($type, $subtype) = (split('/', $head->mime_type, -1), '');
     $self->debug("type = $type, subtype = $subtype");
 
     ### Handle, according to the MIME type:
@@ -1138,18 +1132,19 @@ sub parse_data {
 
     ### Get data as a scalar:
     my $io;
-  switch: while(1) {
-      (!ref($data)) and do {
-	  $io = new IO::Scalar \$data; last switch;
-      };
-      (ref($data) eq 'SCALAR') and do {
-	  $io = new IO::Scalar $data; last switch;
-      };
-      (ref($data) eq 'ARRAY') and do {
-	  $io = new IO::ScalarArray $data; last switch;
-      };
-      croak "parse_data: wrong argument ref type: ", ref($data);
-  }
+
+    if (! ref $data ) {
+        $io = IO::File->new(\$data, '<:');
+    } elsif( ref $data eq 'SCALAR' ) {
+        $io = IO::File->new($data, '<:');
+    } elsif( ref $data eq 'ARRAY' ) {
+	# Unfortunately, if they give us an array, we have to keep
+	# using it.  We don't really want to make a copy.
+	# TODO: I think we're stuck keeping this one for now.
+        $io = IO::ScalarArray->new($data);
+    } else {
+        croak "parse_data: wrong argument ref type: ", ref($data);
+    }
 
     ### Parse!
     return $self->parse($io);
@@ -1162,10 +1157,9 @@ sub parse_data {
 I<Instance method.>
 Takes a MIME-stream and splits it into its component entities.
 
-The INSTREAM can be given as a readable FileHandle, an IO::File,
-a globref filehandle (like C<\*STDIN>),
-or as I<any> blessed object conforming to the IO:: interface
-(which minimally implements getline() and read()).
+The INSTREAM can be given as an IO::File, a globref filehandle (like
+C<\*STDIN>), or as I<any> blessed object conforming to the IO::
+interface (which minimally implements getline() and read()).
 
 Returns the parsed MIME::Entity on success.
 Throws exception on failure.  If the message contained too many
@@ -1175,7 +1169,7 @@ parts (as set by I<max_parts>), returns undef.
 
 sub parse {
     my $self = shift;
-    my $in = wraphandle(shift);    ### coerce old-style filehandles to objects
+    my $in = shift;
     my $entity;
     local $/ = "\n";    ### just to be safe
 
@@ -1247,13 +1241,13 @@ Throws exception on failure.
 
 sub parse_two {
     my ($self, $headfile, $bodyfile) = @_;
-    my @lines;
+    my $data;
     foreach ($headfile, $bodyfile) {
 	open IN, "<$_" or die "$ME: open $_: $!";
-	push @lines, <IN>;
+	$data .= do { local $/; <IN> };
 	close IN or die "$ME: can't close: $!";
     }
-    return $self->parse_data(\@lines);
+    return $self->parse_data($data);
 }
 
 =back
@@ -1482,30 +1476,25 @@ sub output_to_core {
     $self->{MP5_FilerToCore};
 }
 
-#------------------------------
 
-=item tmp_recycling [YESNO]
+=item tmp_recycling
 
-I<Instance method.>
-Normally, tmpfiles are created when needed during parsing, and
-destroyed automatically when they go out of scope.  But for efficiency,
-you might prefer for your parser to attempt to rewind and reuse the
-same file until the parser itself is destroyed.
+I<Instance method, DEPRECATED.>
 
-If YESNO is true (the default), we allow recycling;
-tmpfiles persist until the parser itself is destroyed.
-If YESNO is false, we do not allow recycling;
-tmpfiles persist only as long as they are needed during the parse.
-With no argument, just returns the current setting.
+This method is a no-op to preserve the pre-5.421 API.
+
+The tmp_recycling() feature was removed in 5.421 because it had never actually
+worked.  Please update your code to stop using it.
 
 =cut
 
-sub tmp_recycling {
-    my ($self, $yesno) = @_;
-    $self->{MP5_TmpRecycling} = $yesno if (@_ > 1);
-    $self->{MP5_TmpRecycling};
+sub tmp_recycling 
+{
+	return;
 }
 
+
+
 #------------------------------
 
 =item tmp_to_core [YESNO]
@@ -1641,13 +1630,48 @@ sub new_body_for {
 
 #------------------------------
 
-=item new_tmpfile [RECYCLE]
+=pod
+
+=back
+
+=head2 Temporary File Creation
+
+=over
+
+=item tmp_dir DIRECTORY
+
+I<Instance method.>
+Causes any temporary files created by this parser to be created in the
+given DIRECTORY.
+
+If called without arguments, returns current value.
+
+The default value is undef, which will cause new_tmpfile() to use the
+system default temporary directory.
+
+=cut
+
+sub tmp_dir
+{
+    my ($self, $dirname) = @_;
+    if ( $dirname ) {
+	$self->{MP5_TmpDir} = $dirname;
+    }
+
+    return $self->{MP5_TmpDir};
+}
+
+=item new_tmpfile
 
 I<Instance method.>
 Return an IO handle to be used to hold temporary data during a parse.
-The default uses the standard IO::File->new_tmpfile() method unless
-L<tmp_to_core()|/tmp_to_core> dictates otherwise, but you can override this.
-You shouldn't need to.
+
+The default uses MIME::Tools::tmpopen() to create a new temporary file,
+unless L<tmp_to_core()|/tmp_to_core> dictates otherwise, but you can
+override this.  You shouldn't need to.
+
+The location for temporary files can be changed on a per-parser basis
+with L<tmp_dir()>.
 
 If you do override this, make certain that the object you return is
 set for binmode(), and is able to handle the following methods:
@@ -1661,34 +1685,22 @@ set for binmode(), and is able to handle the following methods:
 
 Fatal exception if the stream could not be established.
 
-If RECYCLE is given, it is an object returned by a previous invocation
-of this method; to recycle it, this method must effectively rewind and
-truncate it, and return the same object.  If you don't want to support
-recycling, just ignore it and always return a new object.
-
 =cut
 
 sub new_tmpfile {
-    my ($self, $recycle) = @_;
+    my ($self) = @_;
 
     my $io;
-    if ($self->{MP5_TmpToCore}) {         ### Use an in-core tmpfile (slow)
-	$io = IO::ScalarArray->new;
-    }
-    else {                                ### Use a real tmpfile (fast)
-					       ### Recycle?
-	if ($self->{TmpRecycling} &&                 ### we're recycling
-	    $recycle &&                              ### something to recycle
-	    $Config{'truncate'} && $io->can('seek')  ### recycling will work
-	    ){
-	    $self->debug("recycling tmpfile: $io");
-	    $io->seek(0, 0) or die "$ME: can't seek: $!";
-	    truncate($io, 0) or die "$ME: can't truncate: $!";
-	}
-	else {                                 ### Return a new one:
-	    $io = tmpopen() or die "$ME: can't open tmpfile: $!\n";
-	    binmode($io) or die "$ME: can't set to binmode: $!";
+    if ($self->{MP5_TmpToCore}) {
+	my $var;
+	$io = IO::File->new(\$var, '+>:') or die "$ME: Can't open in-core tmpfile: $!";
+    } else {
+	my $args = {};
+	if( $self->tmp_dir ) {
+		$args->{DIR} = $self->tmp_dir;
 	}
+	$io = tmpopen( $args ) or die "$ME: can't open tmpfile: $!\n";
+	binmode($io) or die "$ME: can't set to binmode: $!";
     }
     return $io;
 }
@@ -1790,7 +1802,6 @@ Optimum settings:
     extract_nested_messages()  0   (may be slightly faster, but in
 				    general you want it set to 1)
     output_to_core()           0   (will be MUCH faster)
-    tmp_recycling()            1?  (probably, but should be investigated)
     tmp_to_core()              0   (will be MUCH faster)
     use_inner_files()          0   (if tmp_to_core() is 0;
 				    use 1 otherwise)
@@ -1838,7 +1849,6 @@ Optimum settings:
     decode_headers()           *** (no real difference)
     extract_nested_messages()  *** (no real difference)
     output_to_core()           0   (will use MUCH less memory)
-    tmp_recycling()            0?  (promotes faster GC if
 				    tmp_to_core is 1)
     tmp_to_core()              0   (will use MUCH less memory)
     use_inner_files()          *** (no real difference, but set it to 1
@@ -1861,7 +1871,6 @@ Optimum settings:
     extract_nested_messages()  0   (sidesteps problems of bad nested messages,
 				    but often you want it set to 1 anyway).
     output_to_core()           *** (doesn't matter)
-    tmp_recycling()            *** (doesn't matter)
     tmp_to_core()              *** (doesn't matter)
     use_inner_files()          *** (doesn't matter)
 
@@ -1880,7 +1889,6 @@ Optimum settings:
     decode_headers()           *** (doesn't matter)
     extract_nested_messages()  *** (doesn't matter)
     output_to_core()           *** (doesn't matter)
-    tmp_recycling              1   (restricts created files to 1 per parser)
     tmp_to_core()              1
     use_inner_files()          1
 
@@ -1948,7 +1956,7 @@ the temp-file use adds significant overhead.
 
 =item Fuzzing of CRLF and newline on input
 
-RFC-1521 dictates that MIME streams have lines terminated by CRLF
+RFC 2045 dictates that MIME streams have lines terminated by CRLF
 (C<"\r\n">).  However, it is extremely likely that folks will want to
 parse MIME streams where each line ends in the local newline
 character C<"\n"> instead.
@@ -1972,14 +1980,14 @@ each line... I<but this is as it should be>.
 =item Inability to handle multipart boundaries that contain newlines
 
 First, let's get something straight: I<this is an evil, EVIL practice,>
-and is incompatible with RFC-1521... hence, it's not valid MIME.
+and is incompatible with RFC 2046... hence, it's not valid MIME.
 
 If your mailer creates multipart boundary strings that contain
 newlines I<when they appear in the message body,> give it two weeks notice
 and find another one.  If your mail robot receives MIME mail like this,
 regard it as syntactically incorrect MIME, which it is.
 
-Why do I say that?  Well, in RFC-1521, the syntax of a boundary is
+Why do I say that?  Well, in RFC 2046, the syntax of a boundary is
 given quite clearly:
 
       boundary := 0*69<bchars> bcharsnospace
@@ -2021,7 +2029,9 @@ my attention.>
 
 =back
 
+=head1 SEE ALSO
 
+L<MIME::Tools>, L<MIME::Head>, L<MIME::Body>, L<MIME::Entity>, L<MIME::Decoder>
 
 =head1 AUTHOR
 
@@ -2031,10 +2041,4 @@ David F. Skoll (dfs@roaringpenguin.com) http://www.roaringpenguin.com
 All rights reserved.  This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
-
-
-=head1 VERSION
-
-$Revision: 1.19 $ $Date: 2005/12/22 14:56:59 $
-
 =cut
